/*
  作成者: ihara789
  作成日: 2023/12/01
  更新日: 2023/12/14
*/

// ---------- 設定 ----------

// エリア
var canvas_width = 600;
var canvas_height = 400;

const field_wrap = document.querySelector(".field__wrap");
field_wrap.style.width = canvas_width + "px";
field_wrap.style.height = canvas_height + "px";

const info__wrap = document.querySelector(".info__wrap");
info__wrap.style.width = canvas_width + "px";
info__wrap.style.height = canvas_height + "px";

const canvas = document.getElementById('playground');
const ctx = canvas.getContext('2d');
canvas.width = canvas_width;
canvas.height = canvas_height;
// console.log(canvas.width);
// console.log(canvas.height);

const spacer = 5;

// パドル
const paddle_height = 8;
const paddle_width = 150;
var paddle_x = (canvas_width - paddle_width) / 2;
var paddle_moving_size = 5;

// ボール
var ball_radius = 10;
const ball_btm_margin = 5;
var ball_x_speed = 1;
var ball_y_speed = 1;
var ball_x = canvas_width / 2;
var ball_y = canvas_height - ball_radius - paddle_height - spacer * 2;
var change_angle = 0.15;

// ブロック
var block_row_count = 6;
var block_col_count = 4;
var block_width = 110;
var block_height = 20;
var block_padding = 5;
var block_offset_top = 50;
var block_offset_left = 70;
var blocks = [];
for (var c = 0; c < block_col_count; c++) {
    blocks[c] = [];
    for (var r = 0; r < block_row_count; r++) {
        blocks[c][r] = { x: 0, y: 0, status: 1 };
    }
}
var hit = false;

// // アイテム
// var item_width = 10;
// var item_height = 10;

// 入力判定
var pushed_right = false;
var pushed_left = false;

// 描画間隔
var frame_span = 3;

// other configuration
var score = 0;
var lives = 3;
var block_count = 1;
var game_status = false;
var game_going;
const infoElement = document.querySelector(".info__title");
const messageElement = document.querySelector(".info__message");
const scoreElement = document.querySelector(".score__count");
const livesElement = document.querySelector(".lives__count");
const status_btn = document.querySelector(".status_btn");
const reset_btn = document.querySelector(".reset_btn");

const message_mistake = "The Mission failed. Press Continue button or Space Key";
const message_pause = "Pause or Space key to Suspend."
const message_resume = "Resume button or Space key to Resume the Game.";
const message_clear = "Congratulations! You have Cleared All the Blocks.";
const message_gameover = "New Game button or Space key to Restart the Game.";
// const message_nextstage = "Stage Completed.Moving on to the next stage.";

const message_status_btn_start = "Start";
const message_status_btn_mistake = "Continue";
const message_status_btn_pause = "Pause";
const message_status_btn_resume = "Resume";
const message_status_btn_clear = "New Game";

// ---------- 描画 ----------

// エリア
function draw_playground() {
    ctx.beginPath();
    ctx.rect(0, 0, canvas_width, canvas_height);
    ctx.fillStyle = '#2c4f54';
    ctx.fill();
    ctx.closePath();
    var canvas = document.querySelector('canvas');
    var playground_rect = canvas.getBoundingClientRect();
    playground_left = playground_rect.left
    scoreElement.innerHTML = score;
}
// ボール
function draw_ball() {
    ctx.beginPath();
    ctx.arc(ball_x, ball_y, ball_radius, 0, Math.PI * 2);
    ctx.fillStyle = '#fdeff2';
    ctx.fill();
    ctx.closePath();
}
// パドル
function draw_paddle() {
    ctx.beginPath();
    ctx.rect(paddle_x, canvas_height - paddle_height - spacer, paddle_width, paddle_height);
    ctx.fillStyle = "#80aba9";
    ctx.fill();
    ctx.closePath();
    // 左
    ctx.beginPath();
    ctx.rect(paddle_x, canvas_height - paddle_height - spacer, paddle_width / 3, paddle_height);
    ctx.fillStyle = "#93bdbb";
    ctx.fill();
    ctx.closePath();
    //右
    ctx.beginPath();
    ctx.rect(paddle_x + paddle_width * 2 / 3, canvas_height - paddle_height - spacer, paddle_width / 3, paddle_height);
    ctx.fillStyle = "#93bdbb";
    ctx.fill();
    ctx.closePath();
}
// ブロック
function draw_blocks() {
    for (var c = 0; c < block_col_count; c++) {
        for (var r = 0; r < block_row_count; r++) {
            if (blocks[c][r].status == 1) {
                var block_X = (c * (block_width + block_padding)) + block_offset_left;
                var block_Y = (r * (block_height + block_padding)) + block_offset_top;
                blocks[c][r].x = block_X;
                blocks[c][r].y = block_Y;
                ctx.beginPath();
                ctx.rect(block_X, block_Y, block_width, block_height);
                if (0 <= r && r <= 5) {
                    ctx.fillStyle = [
                        '#ff7abc',
                        '#bc7aff',
                        '#7abcff',
                        '#7affbc',
                        '#bcff7a',
                        '#ffbc7a',
                    ][r];
                }
                ctx.fill();
                ctx.closePath();
            }
        }
    }
}

// // アイテム
// function draw_item() {
//     ctx.beginPath();
//     ctx.rect(x, y, item_width, item_height);
//     ctx.fillStyle = "#f4ff29";
//     ctx.fill();
//     ctx.closePath();
// }

//---------------------- 操作 --------------------------

// パドル（キーボード入力） ----------------------
function paddle_KeyDown(arrow_buttons) {
    if (arrow_buttons.key == "ArrowRight") {
        pushed_right = true;
    } else if (arrow_buttons.key == "ArrowLeft") {
        pushed_left = true;
    }
}
function paddle_KeyUp(arrow_buttons) {
    if (arrow_buttons.key == "ArrowRight") {
        pushed_right = false;
    } else if (arrow_buttons.key == "ArrowLeft") {
        pushed_left = false;
    }
}
document.addEventListener("keydown", paddle_KeyDown, false);
document.addEventListener("keyup", paddle_KeyUp, false);

function paddle_controle() {
    if (pushed_right &&
        paddle_x < canvas_width - paddle_width - paddle_moving_size) {
        paddle_x += paddle_moving_size;
    } else if (pushed_left &&
        paddle_x > paddle_moving_size) {
        paddle_x -= paddle_moving_size;
    }
}

// パドル（マウス入力） ----------------------
document.addEventListener("mousemove", function (event) {
    var mouse_x = event.clientX;
    if (playground_left + paddle_width / 2 <= mouse_x &&
        mouse_x <= playground_left + canvas_width - paddle_width / 2) {
        paddle_x = mouse_x - playground_left - paddle_width / 2
    }
});

// 開始・停止（ボタンクリック）---------------------- 
status_btn.addEventListener("click", () => {
    if (block_count == 0 || lives <= 0) {
        window.location.reload();
    } else {
        game_status_switch();
    }
});

// 開始・停止（スペースキー入力）---------------------- 
document.addEventListener("keydown", (event) => {
    if (event.target !== status_btn && event.key === " ") {
        if (block_count == 0 || lives <= 0) {
            window.location.reload();
        } else {
            game_status_switch();
        }
    }
});

// リセット（ボタンクリック）---------------------- 
const resetBtn = document.querySelector(".reset_btn");
resetBtn.addEventListener("click", () => {
    window.location.reload();
});

//---------------------- ボールの動き --------------------------

function ball_bouncy() {
    livesElement.innerHTML = lives;

    if (ball_x - ball_radius <= 0 && ball_x_speed < 0) {
        ball_x_speed = - ball_x_speed;
    } else if (ball_x + ball_radius >= canvas_width && 0 < ball_x_speed) {
        ball_x_speed = - ball_x_speed;
    } else {
        ball_x = ball_x + ball_x_speed;
    }
    if (ball_y - ball_radius <= 0 && ball_y_speed > 0) {
        ball_y_speed = - ball_y_speed;
    } else if (ball_y + ball_radius + paddle_height + spacer >= canvas_height && 0 > ball_y_speed) {
        if (paddle_x < ball_x && ball_x < paddle_x + paddle_width) {
            // Success
            // Change Angle
            ball_y_speed = - ball_y_speed;
            if (0.4 < ball_x_speed || 0.4 < ball_y_speed) {
                if (ball_x < paddle_x + paddle_width / 3) {
                    ball_x_speed = ball_x_speed - change_angle
                    if (0 < ball_x_speed) {
                        // console.log("| L | → | x-a | y+a |");
                        ball_y_speed = ball_y_speed + change_angle
                    } else {
                        // console.log("| L | ← | x-a | y-a|");
                        ball_y_speed = ball_y_speed - change_angle
                    }
                } else if (paddle_x + paddle_width * 2 / 3 < ball_x) {
                    ball_x_speed = ball_x_speed + change_angle
                    if (0 < ball_x_speed) {
                        // console.log("| R | → | x+a | y-a |");
                        ball_y_speed = ball_y_speed - change_angle
                    } else {
                        // console.log("| R | ← | x+a | y+a |");
                        ball_y_speed = ball_y_speed + change_angle
                    }
                }
            }
        } else {
            // Mistake
            lives = lives - 1
            livesElement.innerHTML = lives;
            game_status = false;
            infoElement.textContent = "Mistake";
            infoElement.classList.add("mistake");
            infoElement.classList.remove("on-play");
            
            messageElement.classList.remove("on-play");
            messageElement.innerHTML = message_mistake;
            status_btn.innerHTML = message_status_btn_mistake;

            // 配置リセット
            ball_x = canvas_width / 2;
            ball_y = canvas_height - 30;
            paddle_x = (canvas_width - paddle_width) / 2;
        }
    } else {
        ball_y = ball_y - ball_y_speed;
    }
}

//---------------------- 衝突判定 --------------------------

function attack() {
    for (var c = 0; c < block_col_count; c++) {
        for (var r = 0; r < block_row_count; r++) {
            if (blocks[c][r].status == 1) {
                // 衝突
                if (blocks[c][r].x < ball_x + ball_radius && blocks[c][r].x + block_width > ball_x - ball_radius
                    && blocks[c][r].y < ball_y + ball_radius && blocks[c][r].y + block_height > ball_y - ball_radius) {
                    if (!hit) {
                        // 進行方向の変更（ボールの中心から接触方向を判定）
                        if (blocks[c][r].y > ball_y || blocks[c][r].y + block_height < ball_y) {
                            ball_y_speed = - ball_y_speed;
                        } else if (blocks[c][r].x > ball_x || blocks[c][r].x < ball_x) {
                            ball_x_speed = - ball_x_speed;
                        }
                        hit = true;
                    }
                    blocks[c][r].status = 0
                    score = score + 10;
                    // ステージクリア判定
                    block_count = blocks.flat().filter(block => block.status === 1).length;
                } else {
                    hit = false;
                }
            }
        }
    }
}

//---------------------- 開始 --------------------------

// 実行切替 ---------------------- 
function game_status_switch() {
    if (game_status) {
        // console.log("resume");
        game_status = false;
        clearInterval(game_going);
        infoElement.textContent = "Pause";
        infoElement.classList.add("pause");
        infoElement.classList.remove("on-play");

        messageElement.classList.remove("on-play");
        messageElement.innerHTML = message_resume;
        status_btn.innerHTML = message_status_btn_resume;

    } else if (game_status == false) {
        if (block_count == 0 || lives <= 0) {
            // console.log("clear or gameover");
        } else {
            // console.log("continueable");
            game_status = true;
            game_going = setTimeout(game_draw, frame_span);
            infoElement.textContent = "On Play";
            infoElement.classList.add("on-play");
            infoElement.classList.remove("pause");
            infoElement.classList.remove("mistake");
            
            messageElement.classList.add("on-play");
            messageElement.innerHTML = message_pause;
            status_btn.innerHTML = message_status_btn_pause;
        }
    }
}

function check_game_status() {
    if (game_status) {
        if (block_count == 0) {
            // console.log("clear");
            setTimeout(function () {
                clearTimeout(game_going);
            }, 200);

            infoElement.textContent = "Stage Clear";
            infoElement.classList.add("stage-clear");
            infoElement.classList.remove("on-play");

            // setTimeout(function () {
            //     infoElement.textContent = "Stage Clear";
            //     infoElement.classList.add("stage-clear");
            // }, 2000);

            
            messageElement.classList.remove("on-play");
            messageElement.innerHTML = message_clear;
            status_btn.innerHTML = message_status_btn_clear;
        } else {
            // console.log("start");
            clearTimeout(game_going);
            game_going = setInterval(game_draw, frame_span);
        }
    } else {
        if (lives <= 0) {
            // console.log("game over");
            clearInterval(game_going);

            infoElement.textContent = "Game Over";
            infoElement.classList.add("game-over");
            infoElement.classList.remove("on-play");

            messageElement.classList.remove("on-play");
            messageElement.innerHTML = message_gameover;
            status_btn.innerHTML = message_status_btn_clear;
        } else {
            // console.log("resume");
            clearInterval(game_going);

            ball_x_speed = 1;
            ball_y_speed = 1;
        }
    }
}

//---------------------- 部品呼出 --------------------------

function game_draw() {
    draw_playground();
    draw_ball();
    draw_paddle();
    draw_blocks();
    paddle_controle();
    ball_bouncy();
    attack();
    check_game_status();
}

//---------------------- 実行 --------------------------
function inplay() {
    // game_going を setTimeout() でラップ
    game_going = setTimeout(game_draw, frame_span);
}
inplay();
